# -*- coding: utf-8 -*-
from .command import adb_command


def wifi(enable: bool) -> None:
	"""Turns Wi-Fi on or off."""
	if enable:
		adb_command("shell svc wifi enable")
	else:
		adb_command("shell svc wifi disable")
	
	
def set_proxy(ip: str, port: int) -> None:
	"""enable proxy for phone
	:param ip: Proxy server ip address.
	:param port: Server proxy port.
	"""
	adb_command(f"shell settings put global http_proxy {ip}:{port}")
	
	
def reset_proxy() -> None:
	"""reset proxy settings"""
	adb_command(f"shell settings put global http_proxy :0")


def bluetooth(enable: bool) -> None:
	"""Turns Bluetooth on or off."""
	if enable:
		adb_command("shell service call bluetooth_manager 6")
	else:
		adb_command("shell service call bluetooth_manager 8")


def airplane_mode(enable: bool) -> None:
	"""Turns airplane mode on or off."""
	if enable:
		adb_command("shell settings put global airplane_mode_on 1")
		adb_command("shell am broadcast -a android.intent.action.AIRPLANE_MODE --ez state true")
	else:
		adb_command("shell settings put global airplane_mode_on 0")
		adb_command("shell am broadcast -a android.intent.action.AIRPLANE_MODE --ez state false")


def mobile_data(enable: bool) -> None:
	"""Disables mobile network data transmission on the device."""
	if enable:
		adb_command("shell svc data enable")
	else:
		adb_command("shell svc data disable")


# -*- coding: utf-8 -*-
from .command import adb_command
import time


def get_device_info() -> dict:
    """Retrieves detailed information about the device."""
    device_info = {
        "Model": adb_command('shell getprop ro.product.model').strip(),
        "Android Version": adb_command('shell getprop ro.build.version.release').strip(),
        "SDK Version": adb_command('shell getprop ro.build.version.sdk').strip(),
        "Device": adb_command('shell getprop ro.product.device').strip(),
        "Manufacturer": adb_command('shell getprop ro.product.manufacturer').strip(),
        "Board": adb_command('shell getprop ro.product.board').strip(),
        "Hardware": adb_command('shell getprop ro.product.hardware').strip(),
        "Serial": adb_command('shell getprop ro.serialno').strip(),
        "Build ID": adb_command('shell getprop ro.build.display.id').strip(),
        "Fingerprint": adb_command('shell getprop ro.build.fingerprint').strip(),
        "Host": adb_command('shell getprop ro.build.host').strip(),
        "Time": adb_command('shell getprop ro.build.date').strip(),
    }
    return device_info


def get_apps() -> list:
    """Gets a list of installed applications on the device.
    :return: List of applications
    """
    
    result = adb_command("shell pm list packages -3")
    return [line.replace('package:', '').strip() for line in result.splitlines() if line]


def run_app(package_name: str):
    """Launches an application by package name.
    :return: class for application control
    """
    app = Application(package_name)
    app.launch()
    return app


def stop_app(package_name) -> None:
    """Stops an application by package name."""
    adb_command(f"shell am force-stop {package_name}")
        
        
class Application:
    def __init__(self, package_name):
        self.package_name = package_name

    def launch(self) -> None:
        """Launches an application"""
        adb_command(f"shell monkey -p {self.package_name} -c android.intent.category.LAUNCHER 1")
        
    def close(self) -> None:
        """Stops the application."""
        adb_command(f"shell am force-stop {self.package_name}")
        
    def restart(self) -> None:
        """Reloads the application."""
        self.close()
        time.sleep(1)
        self.launch()

    def __repr__(self):
        return f"Application(package_name='{self.package_name}')"


# -*- coding: utf-8 -*-
import time
import cv2
import os
import numpy as np

from .command import adb_command


def screenshot(filename="screen.png") -> str:
    """Takes a screenshot and saves it to disk."""
    adb_command("exec-out screencap -p > {}".format(filename))
    return filename


def search_image(image_file, attempts=1, wait_time=0) -> tuple:
    """
    Searches for an image on the screen, taking several screenshots.

    :param image_file: Name of the image file to search for.
    :param attempts: Number of search attempts.
    :param wait_time: Time in seconds between attempts.
    :return: Coordinates of the found image or None if not found.
    """

    if not os.path.isfile(image_file):
        raise FileNotFoundError(f"Файл {image_file} не найден.")
    
    for attempt in range(attempts):
        screenshot_path = screenshot('find.png')
        coordinates = _find_image(screenshot_path, image_file)
        os.remove('find.png')
    
        if coordinates:
            return coordinates[0]
    
        time.sleep(wait_time)

    return ()
    

def _find_image(screenshot_path, image_path):
    screenshot = cv2.imread(screenshot_path)
    image = cv2.imread(image_path)

    screenshot_gray = cv2.cvtColor(screenshot, cv2.COLOR_BGR2GRAY)
    image_gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

    result = cv2.matchTemplate(screenshot_gray, image_gray, cv2.TM_CCOEFF_NORMED)
    threshold = 0.8
    locations = np.where(result >= threshold)

    points = []
    h, w = image_gray.shape

    for pt in zip(*locations[::-1]):
        center_x = pt[0] + w // 2
        center_y = pt[1] + h // 2
        points.append((center_x, center_y))

    return points


# -*- coding: utf-8 -*-
from .command import adb_command


def tap(x: int, y: int) -> None:
	"""Clicks on the screen at the specified coordinates."""
	adb_command(f"shell input tap {x} {y}")


def swipe(start_x: int, start_y: int, end_x: int, end_y: int, duration=999) -> None:
	"""swipe from one place to another."""
	adb_command(f"shell input swipe {start_x} {start_y} {end_x} {end_y} {duration}")
	
	
def input_text(text: str) -> None:
	"""Enter some message into the text field for input."""
	adb_command(f'shell input text "{text.replace(" ", "%s")}"')
	
	
def screen() -> None:
	"""Turns phone screen on or off."""
	adb_command("shell input keyevent 26")
	

def reboot() -> None:
	"""Reboot phone."""
	adb_command("shell reboot")
	
	
def power_off() -> None:
	"""Reboot phone."""
	adb_command("shell reboot -p")
	
	
def toast(message: str) -> None:
	"""Display a message on the screen."""
	adb_command(f'shell settings put global toast "{message}"')
	
	
def send_notification(text):
    """Sends a notification to the device with the specified text."""
    command = f'shell cmd notification post default_channel {text}'
    adb_command(command)
	

def set_brightness(value: int) -> None:
    """Sets the screen brightness (from 0 to 255)."""
    if 0 <= value <= 255:
        adb_command(f'shell settings put system screen_brightness {value}')
    else:
        raise ValueError("Brightness should be between 0 and 255.")
	
	
def set_volume(volume: int) -> None:
    """Sets the sound level (from 0 to 15)."""
    if 0 <= volume <= 15:
        adb_command(f'shell media volume --set {volume}')
    else:
        raise ValueError("The sound level should be between 0 and 15.")


import shutil

from .command import set_device
from .control import *
from .screen import *
from .action import *
from .servis import *


class Phone:
    def __init__(self, adb_path=None, device=None):
        
        if device:
            set_device(device)
            
        if adb_path:
            os.environ['PATH'] += os.pathsep + adb_path

        if not shutil.which("adb"):
            raise EnvironmentError("ADB not found. Make sure ADB is installed and added to PATH.")
        
        
__all__ = ["Phone"]


import subprocess

device_id = None


def adb_command(command, shell=True, capture_output=True, text=True) -> str:
    """Method for executing an ADB command and returning the result."""
    
    if not device_id:
        result = subprocess.run(f"adb {command}", shell=shell, capture_output=capture_output, text=text)
    else:
        result = subprocess.run(f"adb -s {device_id} {command}", shell=shell, capture_output=capture_output, text=text)
        
    if result.returncode == 0:
        return result.stdout.strip()
    else:
        raise RuntimeError(f"ADB command failed: {result.stderr.strip()}")
    
    
def set_device(device):
    global device_id
    device_id = device